

#include "cah.h"
#include <cmath>


CAH::CAH(QString data, QVector<double>pond, uint dst, uint cri, uint prec) {
	this->step = 0;
	this->ic = 0;

	this->loadData(data);

	this->pond = pond;
	this->pond.resize(this->items.size());

	this->dst = dst;
	this->cri = cri;
	this->prec = prec;
}


void CAH::loadData(const QString & data) {
	QStringList rows = data.split('\n');
        QStringList first;
	uint width = 0, height = 0;

        if(!rows.size())
                return;

        first = rows.first().split(" ");
        rows.removeAt(0);

        if(first.size() != 2)
                return;

	width = first.at(0).toUInt();
	height = first.at(1).toUInt();

	for(this->ic = 0; this->ic < height; ++this->ic) {
		this->clusters.append(Cluster(this->ic, 0));
		this->clusters[this->ic].append(Item(this->ic));
	}

        for (uint i = 0; i < height; ++i) {
                QStringList cols = rows.at(i).split(";");
                for (uint j = 0; j < width; ++j) {
                        double value = cols.at(j).toDouble();
			this->clusters[i][0].append(value);
                }
		items.append(this->clusters[i][0]);
        }
}

QString CAH::getData() {
	QString str;
	str.append("Classification Ascendante Hirarchique\n");
	str.append("-------\n");
	str.append("Donnes :\n");

	str.append("\t");
	for(int j = 0; j < items[0].size(); ++j)
		str.append(QString("V%1\t").arg(j));

	str.append("\n");

	for(int i = 0; i < items.size(); ++i) {
		str.append(items[i].getName());
		str.append("\t");
		for(int j = 0; j < items[i].size(); ++j) {
			str.append(QString::number(items[i][j], 'f', this->prec));
			str.append("\t");
		}
		str.append("\n");
	}

	str.append("\nPoids\n");
	for(int j = 0; j < items[0].size(); ++j) {
		str.append(QString("V%1: ").arg(j));
		str.append(QString::number(this->pond[j], 'f', this->prec));
		str.append("\n");
	}
	str.append("\n");

	return str;
}

double CAH::calccri(const Cluster & a, const Cluster & b) {
	double result;
	switch(this->cri) {
	case CRI_MIN:
	{
		double min = -1;
		for(int i = 0; i < a.size(); ++i) {
			for(int j = 0; j < b.size(); ++j) {
				double dst = calcdst(a[i], b[j]);
				if(min < 0 || min > dst)
					min = dst;
			}
		}
		result = min;
	}
		break;
	case CRI_MOY:
	{
		double moy = 0;
		for(int i = 0; i < a.size(); ++i) {
			for(int j = 0; j < b.size(); ++j) {
				moy += calcdst(a[i], b[j]);
			}
		}
		result = moy / (a.size() * b.size());
	}
		break;
	case CRI_MAX:
	default:
	{
		double max = 0;
		for(int i = 0; i < a.size(); ++i) {
			for(int j = 0; j < b.size(); ++j) {
				double dst = calcdst(a[i], b[j]);
				if(dst > max)
					max = dst;
			}
		}
		result = max;
	}
		break;
	}
	return result;
}

double CAH::calcdst(const Item & a, const Item & b) {
	double result = 0;
	uint size = a.size();

	for(uint i = 0; i < size; ++i) {
		result += this->pond[i] * (a[i] - b[i]) * (a[i] - b[i]);
	}

	result = sqrt(result);

	if(this->dst == DST_EUCLI2)
		result *= result;

	return result;
}

QString CAH::run() {
	QVector< QVector < double > > cri;
	QString str;
	uint size = this->clusters.size();

	str.append("-----\n");
	str.append(QString("Etape %1:\n").arg(this->step));

	/* Calcul */
	cri.resize(size);
	for(uint i = 0; i < size; ++i) {
		cri[i].resize(size);
		for(uint j = 0; j < size; ++j) {
			cri[i][j] = calccri(this->clusters[i], this->clusters[j]);
		}
	}


	/* "Affichage" */
	str.append("Tableau des dissimilarites\n");
	str.append("\t");

	for(uint j = 0; j < size; ++j) {
		str.append(this->clusters[j].getName());
		str.append("\t");
	}

	str.append("\n");
	for(uint i = 0; i < size; ++i) {
		str.append(this->clusters[i].getName());
		str.append("\t");
		cri[i].resize(size);
		for(uint j = 0; j < size; ++j) {
			str.append(QString::number(cri[i][j], 'f', this->prec));
			str.append("\t");
		}
		str.append("\n");
	}


	/* Trouver le minimum */
	double min = -1;
	uint imin = 0;
	uint jmin = 0;
	for(uint i = 0; i < size; ++i) {
		for(uint j = 0; j < size; ++j) {
			if(i != j && (cri[i][j] < min || min == -1)) {
				min = cri[i][j];
				imin = i, jmin = j;
			}
		}
	}


	/* "Affichage" */
	str.append("Dissimilarit minimale:\n");
	str.append(this->clusters[imin].getName() + " - " +this->clusters[jmin].getName());
	str.append(": " + QString::number(min, 'f', this->prec));
	str.append("\n");

	/* Regrouper */
	Cluster c(this->ic, min);
	this->ic++;

	for(int i = 0; i < this->clusters[imin].size(); ++i) {
		c.append(this->clusters[imin][i]);
	}
	for(int i = 0; i < this->clusters[jmin].size(); ++i) {
		c.append(this->clusters[jmin][i]);
	}
	c.addChild(this->clusters[jmin]);
	c.addChild(this->clusters[imin]);

	this->clusters.removeAt(imin);
	if(imin < jmin)
		this->clusters.removeAt(jmin-1);
	else
		this->clusters.removeAt(jmin);

	this->clusters.append(c);


	/* "Affichage" */
	str.append("Nouveau: " + c.getName() + " (" + c.getValues() + ")");
	str.append("\n");


	this->step++;
	return str;
}


QList<Cluster>
CAH::getClusters() {
	return this->clusters;
}
