#include <QtGui>

#include "mainwindow.h"

MainWindow::MainWindow()
{
	matrix = new QTableWidget();
	setCentralWidget(matrix);

	createActions();
	createMenus();

	readSettings();

	connect(matrix, SIGNAL(cellChanged(int, int)),
		this, SLOT(matrixWasModified()));

	setCurrentFile("");
	newFile();
}

void
 MainWindow::closeEvent(QCloseEvent * event)
{
	if (maybeSave()) {
		writeSettings();
		event->accept();
	} else {
		event->ignore();
	}
}

void MainWindow::newFile()
{
	if (maybeSave()) {
		uint height = (uint) QInputDialog::getInteger (this, "Nombre de lignes", "Nombre de lignes", 5, 0);
		uint width = (uint) QInputDialog::getInteger (this, "Nombre de colonnes", "Nombre de colonnes", 5, 0);
		resizeTable(width, height);
		setCurrentFile("");
	}
}

void MainWindow::open()
{
	if (maybeSave()) {
		QString fileName = QFileDialog::getOpenFileName(this);
		if (!fileName.isEmpty())
			loadFile(fileName);
	}
}

bool MainWindow::save()
{
	if (curFile.isEmpty()) {
		return saveAs();
	} else {
		return saveFile(curFile);
	}
}

bool MainWindow::saveAs()
{
	QString fileName = QFileDialog::getSaveFileName(this);
	if (fileName.isEmpty())
		return false;

	return saveFile(fileName);
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("A propos de MatrixEditor"),
			   tr
			   ("<b>MatrixEditor</b> permet de crer des matrices, "
			    "Youpi ! "));
}

void MainWindow::matrixWasModified()
{
	matrixIsModified = true;
	setWindowModified(true);
}

void MainWindow::createActions()
{
	newAct = new QAction(tr("&Nouveau"), this);
	newAct->setShortcut(tr("Ctrl+N"));
	newAct->setStatusTip(tr("Creer une nouvelle matrice"));
	connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

	openAct = new QAction(tr("&Ouvrir..."), this);
	openAct->setShortcut(tr("Ctrl+O"));
	openAct->setStatusTip(tr("Ouvrir une matrice"));
	connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

	saveAct = new QAction(tr("Enregi&strer"), this);
	saveAct->setShortcut(tr("Ctrl+S"));
	saveAct->setStatusTip(tr("Enregistrer la matrice"));
	connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

	saveAsAct = new QAction(tr("Enr&egistrer sous..."), this);
	saveAsAct->
	    setStatusTip(tr("Enregistrer la matrice sous un nouveau nom"));
	connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

	exitAct = new QAction(tr("&Quitter"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setStatusTip(tr("Quitter l'application"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	aboutAct = new QAction(tr("&A propos"), this);
	aboutAct->setStatusTip(tr("Show the application's About box"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	aboutQtAct = new QAction(tr("A propos de &Qt"), this);
	aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
	connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void MainWindow::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&Fichier"));
	fileMenu->addAction(newAct);
	fileMenu->addAction(openAct);
	fileMenu->addAction(saveAct);
	fileMenu->addAction(saveAsAct);
	fileMenu->addSeparator();
	fileMenu->addAction(exitAct);

	menuBar()->addSeparator();

	helpMenu = menuBar()->addMenu(tr("A&ide"));
	helpMenu->addAction(aboutAct);
	helpMenu->addAction(aboutQtAct);
}

void MainWindow::readSettings()
{
	QSettings settings("iksaif", "MatrixEditor");
	QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
	QSize size = settings.value("size", QSize(400, 400)).toSize();
	resize(size);
	move(pos);
}

void MainWindow::writeSettings()
{
	QSettings settings("iksaif", "MatrixEditor");
	settings.setValue("pos", pos());
	settings.setValue("size", size());
}

bool MainWindow::maybeSave()
{
	if (matrixIsModified) {
		QMessageBox::StandardButton ret;
		ret = QMessageBox::warning(this, tr("MatrixEditor"),
					   tr
					   ("Le document  t modifi,\n"
					    "voulez vous enregistrer les modifications ?"),
					   QMessageBox::Save |
					   QMessageBox::Discard |
					   QMessageBox::Cancel);
		if (ret == QMessageBox::Save)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void MainWindow::loadFile(const QString & fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("MatrixEditor"),
				     tr("Impossible de lire %1:\n%2.").
				     arg(fileName).arg(file.
						       errorString()));
		return;
	}

	QTextStream in(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	if(!fillTable(in.readAll())) {
		matrix->clear();
		QMessageBox::warning(this, tr("MatrixEditor"),
				     tr("Mauvaix format de fichier"));
		QApplication::restoreOverrideCursor();
		return;
	}
	QApplication::restoreOverrideCursor();
	setCurrentFile(fileName);
}

void MainWindow::resizeTable(const uint width, const uint height) {
	QStringList colsLabels;
	QStringList rowsLabels;

	for(uint i = 0; i < width; ++i)
		colsLabels << QString("V%1").arg(i);

	for(uint i = 0; i < height; ++i)
		rowsLabels << QString("A%1").arg(i);

	matrix->setRowCount(height);
	matrix->setVerticalHeaderLabels(rowsLabels);

	matrix->setColumnCount(width);
	matrix->setHorizontalHeaderLabels(colsLabels);
}

bool MainWindow::fillTable(const QString & data)
{
	QStringList rows = data.split('\n');
	QStringList first;

	if(!rows.size())
		return false;

	first = rows.first().split(" ");
	rows.removeAt(0);

	if(first.size() != 2)
		return false;

	resizeTable(first.at(0).toUInt(), first.at(1).toUInt());

	for (int i = 0; i < rows.size(); ++i) {
		QStringList cols = rows.at(i).split(";");
		for (int j = 0; j < cols.size(); ++j) {
			double value = cols.at(j).toDouble();
			QTableWidgetItem *item = new QTableWidgetItem(tr("%1").arg(value));
			item->setFlags(Qt::ItemIsEnabled|Qt::ItemIsEditable|Qt::ItemIsSelectable);
			matrix->setItem(i, j, item);
		}
	}
	return true;
}

bool MainWindow::saveFile(const QString & fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("MatrixEditor"),
				     tr("Impossible d'crire %1:\n%2.").
				     arg(fileName).arg(file.errorString()));
		return false;
	}

	QTextStream out(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << exportTable();
	QApplication::restoreOverrideCursor();

	setCurrentFile(fileName);
	return true;
}

QString MainWindow::exportTable() {
	QString str;
	uint width = matrix->columnCount();
	uint height = matrix->rowCount();

	str.append(QString("%1 %2\n").arg(width).arg(height));

	for (uint i = 0; i < height; ++i) {
		QStringList row;
		for (uint j = 0; j < width; ++j) {
			QTableWidgetItem *item = matrix->item(i, j);
			if(item)
				row << item->text();
			else
				row << QString("0");
		}
		str.append(row.join(";"));
		str.append("\n");
	}
	return str;
}

void MainWindow::setCurrentFile(const QString & fileName)
{
	curFile = fileName;
	matrixIsModified = false;
	setWindowModified(false);

	QString shownName;
	if (curFile.isEmpty())
		shownName = "sans_titre.txt";
	else
		shownName = strippedName(curFile);

	setWindowTitle(tr("%1[*] - %2").arg(shownName).
		       arg(tr("MatrixEditor")));
}

QString MainWindow::strippedName(const QString & fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}
